@@ -14,6 +14,7 @@
 #endif
 
 #include <iostream>
+#include <fstream>
 #include <pthread.h>
 #include <qapplication.h>
 #include <qpushbutton.h>
@@ -30,7 +31,13 @@
 #include <sys/socket.h>
 #include <arpa/inet.h>
 #include <netinet/in.h>
-#include <qstatusbar.h>
+
+#ifdef WITH_KDE
+  #include <kstatusbar.h>
+#else
+  #include <qstatusbar.h>
+#endif
+
 #include "getservice.h"
 
 #include <stdlib.h>
@@ -233,8 +240,11 @@ void NWThread::run()
  *  Constructs a fireflier_client which is a child of 'parent', with the
  *  name 'name' and widget flags set to 'f'
  */
-fireflier_client::fireflier_client( QWidget* parent,  const char* name, WFlags fl )
-    : fireflier_main( parent, name, fl )
+#ifdef WITH_KDE
+fireflier_client::fireflier_client( QWidget* parent,  const char* name, WFlags fl ) : fireflier_main( )
+#else
+fireflier_client::fireflier_client( QWidget* parent,  const char* name, WFlags fl ) : fireflier_main( parent, name, fl )
+#endif
 {
     packetAvailable=0;
 }
@@ -879,11 +889,12 @@ void fireflier_client::del_iptrule_clicked()
 
 void usage()
 {
-    cout << "Usage: fireflier-client-qt -s SERVER_IP -p SERVER_PORT -r" << endl;
+    cout << "Usage: fireflier-client-qt -s SERVER_IP -p SERVER_PORT -r -a filename" << endl;
     cout << "  Default: fireflier-client-qt -s 127.0.0.1 -p 1133" << endl;
     cout << "  -s ... connect to this ip" << endl;
     cout << "  -p ... connect to this port" << endl;
     cout << "  -r ... auto popup (only used with kde client)" << endl;
+    cout << "  -a ... get username/password from this file instead of asking (security risk - see AUTOLOGIN)" << endl;
 }
 
 void deinit()
@@ -898,9 +909,10 @@ int main(int argc, char **argv)
 
 	string serverip="";
 	string serverport="";
+    string autologin="";
 	int needusage=0;
     char option=0;
-	while ((option=getopt(argc, argv, "s:p:r"))!=-1) // using this the first time, hope it works this way :)
+	while ((option=getopt(argc, argv, "s:p:a:r"))!=-1) // using this the first time, hope it works this way :)
 	{
 		switch (option)
 		{
@@ -910,6 +922,9 @@ int main(int argc, char **argv)
 		case 'p':
             serverport=argv[optind-1];
 			break;
+		case 'a':
+            autologin=argv[optind-1];
+			break;
 		case 'r':
             popup=true;
             break;
@@ -922,12 +937,13 @@ int main(int argc, char **argv)
 #ifdef WITH_KDE
 	KAboutData aboutData( "fireflier-client-kde", I18N_NOOP("fireflier-client-kde"),
 						 "1.1.4", "FireFlier Client", KAboutData::License_GPL,
-						 "(c) 2002, Martin Maurer and Peter Maurer");
+						 "(c) 2002, Martin Maurer");
 	aboutData.addAuthor("Martin Maurer",0, "fireflier@gibraltar.at");
 
 	static KCmdLineOptions options[] = {
 		{ "s <argument>", I18N_NOOP("Server where this client should connect to"), "127.0.0.1" },
 		{ "p <argument>", I18N_NOOP("Port this client should connect to"), "1133" },
+		{ "a <argument>", I18N_NOOP("get username/password from this file instead of asking (security risk - see AUTOLOGIN)"), "" },
 		{ "r", I18N_NOOP("Autoraise window on new packet"), "false" },
 		{ 0, 0, 0}
 	};
@@ -995,7 +1011,25 @@ int main(int argc, char **argv)
     qt.start();
     atexit(deinit);
 
-	cl->show();
+	string autouser="";
+    string autopass="";
+	if (autologin!="")
+	{
+        char temp[50];
+		ifstream in(autologin.c_str());
+		if (in.is_open())
+		{
+			in.getline(temp, 49);
+			temp[49]=0;
+            autouser=temp;
+			in.getline(temp, 49);
+			temp[49]=0;
+            autopass=temp;
+		}
+	}
+
+	if (autopass=="")
+		cl->show();
     login_dialog ld;
 	char *username=getlogin();
     if (username)
@@ -1008,30 +1042,43 @@ int main(int argc, char **argv)
         else
 			cl->statusBar()->message("login error - try again");
 
-		ld.exec();
-		if (ld.result()==QDialog::Rejected)
-			exit(1);
+		if ((autopass=="") || (loginagain))
+		{
+			ld.exec();
+			if (ld.result()==QDialog::Rejected)
+				exit(1);
+		}
   		cl->statusBar()->message("sending login and password");
 		kApp->processEvents();
         // authenticated is set to -1
-        // as soon as login result is received, authenticated is set again
-		sendLogin(ld.input_username->text(), ld.input_password->text());
+		// as soon as login result is received, authenticated is set again
+		if ((autopass!="") && !loginagain)
+			sendLogin(autouser.c_str(), autopass.c_str());
+        else
+			sendLogin(ld.input_username->text(), ld.input_password->text());
 		while (((authenticated==-1) || (authenticated==2)) && (connected))
 		{
 			kApp->processEvents();
             sleep(1);
 		}
 //			sleep(1);
-        if (!authenticated)
+		if (!authenticated)
+		{
 			loginagain=true;
+			if (!cl->isVisible())
+				cl->show();
+		}
 	} while ((authenticated!=1) && (connected));
 	cl->statusBar()->message("logged in successfully");
+    if (popup)
+        cl->hide();
 
 	if (!connected)
         return 1;
 
 #ifdef WITH_KDE
-	KWin::appStarted();
+//  deprecated
+//	KWin::appStarted();
 	kApp->exec();
 #else
 	qApp->exec();
