@@ -1,10 +1,11 @@
 #include "backend.h"
 
 #include <openssl/pem.h>
+#include <netinet/in.h>
+
 
 // some important outputs for debugging
-#define DEBUG 1
-
+#define DEBUG 0
 // further outputs (entering and leaving procedures ...)
 #define ERRORDEBUG 0
 
@@ -107,15 +108,109 @@ char *int_to_ipaddr(u_int32_t number, char *temp, int maxlen)
     return temp;
 }
 
+// Copied from iptables.c (error including iptables.h).
+
+/*
+ * Author: Paul.Russell@rustcorp.com.au and mneuling@radlogic.com.au
+ *
+ * (C) 2000-2002 by the netfilter coreteam <coreteam@netfilter.org>:
+ * 		    Paul 'Rusty' Russell <rusty@rustcorp.com.au>
+ * 		    Marc Boucher <marc+nf@mbsi.ca>
+ * 		    James Morris <jmorris@intercode.com.au>
+ * 		    Harald Welte <laforge@gnumonks.org>
+ * 		    Jozsef Kadlecsik <kadlec@blackhole.kfki.hu>
+ *
+ *	This program is free software; you can redistribute it and/or modify
+ *	it under the terms of the GNU General Public License as published by
+ *	the Free Software Foundation; either version 2 of the License, or
+ *	(at your option) any later version.
+ */
+
+
+int
+string_to_number(const char *s, unsigned int min, unsigned int max,
+		 unsigned int *ret)
+{
+	long number;
+	char *end;
+
+	/* Handle hex, octal, etc. */
+	errno = 0;
+	number = strtol(s, &end, 0);
+	if (*end == '\0' && end != s) {
+		/* we parsed a number, let's see if we want this */
+		if (errno != ERANGE && min <= number && number <= max) {
+			*ret = number;
+			return 0;
+		}
+	}
+	return -1;
+}
+
+
+struct in_addr *
+dotted_to_addr(const char *dotted)
+{
+	static struct in_addr addr;
+	unsigned char *addrp;
+	char *p, *q;
+	unsigned int onebyte;
+	int i;
+	char buf[20];
+
+	/* copy dotted string, because we need to modify it */
+	strncpy(buf, dotted, sizeof(buf) - 1);
+	addrp = (unsigned char *) &(addr.s_addr);
+
+	p = buf;
+	for (i = 0; i < 3; i++) {
+		if ((q = strchr(p, '.')) == NULL)
+			return (struct in_addr *) NULL;
+
+		*q = '\0';
+		if (string_to_number(p, 0, 255, &onebyte) == -1)
+			return (struct in_addr *) NULL;
+
+		addrp[i] = (unsigned char) onebyte;
+		p = q + 1;
+	}
+
+	/* we've checked 3 bytes, now we check the last one */
+	if (string_to_number(p, 0, 255, &onebyte) == -1)
+		return (struct in_addr *) NULL;
+
+	addrp[3] = (unsigned char) onebyte;
+
+	return &addr;
+}
+
+// end included from iptables.c 
 
 // send current packet information to client
 void sendpacket()
 {
 	char *buffer;
-	int count;
+	int count, length;
+
+	unsigned short *addr_length;
+	unsigned long *addr_packet_id;
+	unsigned long *addr_arrived;
+	unsigned short int *addr_len;
+	unsigned long int *addr_ip_src;
+	unsigned long int *addr_ip_dst;
+	unsigned char *addr_protocol;
+	unsigned short int *addr_port_src; 
+	unsigned short int *addr_port_dst; 
+	unsigned short int *addr_tcp_flags;
+	unsigned char *addr_icmp_type;
+	unsigned char *addr_hook;	
+	unsigned char *addr_mac_addrlen;
+
+
 	if (!ssl_auth) return;
 	if (!currentPacket) return;
 	buffer=new char[sizeof(struct packet)-4+3+((currentPacket->programname)?strlen(currentPacket->programname)+1:1)];
+
 	if (!buffer)
 	{
 		cerr << "LINE " << __LINE__ << ": COULD NOT ALLOCATE MEMORY!!!" << endl;
@@ -123,76 +218,97 @@ void sendpacket()
 		return;
 	}
 
+	addr_length = (unsigned short *) (buffer+1);
+	addr_packet_id = (unsigned long *) (buffer +3);
+	addr_arrived = (unsigned long *) (buffer +7);
+	addr_len = (unsigned short *) (buffer +11);
+	addr_ip_src = (unsigned long *) (buffer +13);
+	addr_ip_dst = (unsigned long *) (buffer +17);
+	addr_protocol = (unsigned char *) (buffer +21);
+	addr_port_src = (unsigned short *) (buffer +22);
+	addr_port_dst = (unsigned short *) (buffer +24);
+	addr_tcp_flags = (unsigned short *) (buffer +26);
+	addr_icmp_type = (unsigned char *) (buffer +28);
+	addr_hook = (unsigned char *) (buffer +29);
+	addr_mac_addrlen = (unsigned char *) (buffer +30);
+
+
+
 	buffer[0]=0;
-	buffer[3]=(currentPacket->packet_id>>24)&255;  // packet_id
-	buffer[4]=(currentPacket->packet_id>>16)&255;
-	buffer[5]=(currentPacket->packet_id>>8)&255;
-	buffer[6]=(currentPacket->packet_id)&255;
+	// unsigned long packet_id
+	*addr_packet_id=htonl( currentPacket->packet_id );  // packet_id
 
-	buffer[7]=(currentPacket->arrived>>24)&255;  // arrived
-	buffer[8]=(currentPacket->arrived>>16)&255;
-	buffer[9]=(currentPacket->arrived>>8)&255;
-	buffer[10]=(currentPacket->arrived)&255;
+	// unsigned long arrived
+	*addr_arrived=htonl( currentPacket->arrived );  // arrived
 
-	buffer[11]=(currentPacket->len>>8)&255; // len
-	buffer[12]=(currentPacket->len)&255;
+	// unsigned short int len
+	*addr_len=htons( currentPacket->len ); // len
 
-	buffer[13]=(currentPacket->ip_src>>24)&255; // source ip
-	buffer[14]=(currentPacket->ip_src>>16)&255;
-	buffer[15]=(currentPacket->ip_src>>8)&255;
-	buffer[16]=(currentPacket->ip_src)&255;
+	// unsigned long ip_src; could be char[4]: no htonl.
+	*addr_ip_src = currentPacket->ip_src;
 
-	buffer[17]=(currentPacket->ip_dst>>24)&255; // destination ip
-	buffer[18]=(currentPacket->ip_dst>>16)&255;
-	buffer[19]=(currentPacket->ip_dst>>8)&255;
-	buffer[20]=(currentPacket->ip_dst)&255;
+	// unsigned long ip_dst; could be char[4]: no htonl.
+	*addr_ip_dst = currentPacket->ip_dst;
 
-	buffer[21]=currentPacket->protocol; // protocol
+	// unsigned int protocol:8;
+	// original protocol type is char....
+	*addr_protocol= (char) currentPacket->protocol; // protocol
 
-	buffer[22]=(currentPacket->port_src>>8)&255; // source port
-	buffer[23]=(currentPacket->port_src)&255;
+	// unsigned short int port_src; 
+	*addr_port_src=htons( currentPacket->port_src ); // source port
 
-	buffer[24]=(currentPacket->port_dst>>8)&255; // destination port
-	buffer[25]=(currentPacket->port_dst)&255;
+	// unsigned short int port_dst; 
+	*addr_port_dst=htons( currentPacket->port_dst ); // destination port
 
-	buffer[26]=(currentPacket->tcp_flags>>8)&255; // tcp flags
-	buffer[27]=(currentPacket->tcp_flags)&255;
+	// unsigned short int tcp_flags;
+	*addr_tcp_flags=htons( currentPacket->tcp_flags ); // tcp flags
 
-	buffer[28]=(currentPacket->icmp_type); // icmp_type
+	// unsigned int icmp_type:8;
+	// original icmp type is char....
+	*addr_icmp_type=currentPacket->icmp_type ; // icmp_type
 
-	buffer[29]=currentPacket->hook; // firewall chain
+	// unsigned int hook;	
+	// WARNING: used first byte only !!
+	*addr_hook= currentPacket->hook; // firewall chain
 
-	buffer[30]=currentPacket->mac_addrlen;  // mac address length
+	// unsigned int mac_addrlen:8;
+	// original mac addrlen was char.
+	*addr_mac_addrlen= currentPacket->mac_addrlen ;  // mac address length
 
+	// unsigned char mac_addr[8];
 	memcpy(buffer+31, currentPacket->mac_addr, 8); // mac address
 
+	// unsigned char interface_in[IFNAMSIZ];
 	memcpy(buffer+39, currentPacket->interface_in, IFNAMSIZ); // incoming interface
 
+	// unsigned char interface_out[IFNAMSIZ];
 	memcpy(buffer+39+IFNAMSIZ, currentPacket->interface_out, IFNAMSIZ); // outgoing interface
 
 	if (currentPacket->programname) // program using localport/localip of this packet
 	{
-		buffer[39+2*IFNAMSIZ]=strlen(currentPacket->programname)+1;
+	        // apparently on 1 byte.
+		buffer[39+2*IFNAMSIZ]=(char) strlen(currentPacket->programname)+1;
 		memcpy(buffer+39+2*IFNAMSIZ+1, currentPacket->programname, strlen(currentPacket->programname)+1);
-		buffer[1]=(39+2*IFNAMSIZ+strlen(currentPacket->programname)+2)>>8;
-		buffer[2]=(39+2*IFNAMSIZ+strlen(currentPacket->programname)+2)&255;
+		length= 39+2*IFNAMSIZ+strlen(currentPacket->programname)+2;
 	}
 	else
 	{
 		buffer[39+2*IFNAMSIZ]=0;
-		buffer[1]=(39+2*IFNAMSIZ+1)>>8;
-		buffer[2]=(39+2*IFNAMSIZ+1)&255;
+		length= 39+2*IFNAMSIZ+1;
 	}
 
+	// aparently on 2 bytes.
+	*addr_length=htons( (short ) length );
+
 	// send the data
-	if ((count=SSL_write(ssl_auth, buffer, 40+2*IFNAMSIZ+((currentPacket->programname)?strlen(currentPacket->programname)+1:0)))<=-1)
+	if ((count=SSL_write(ssl_auth, buffer, length))<=-1)
 	{
 		// on error disconnect client
 		//		clientConnected=0;
 		free_ssl_auth();
 		cerr << "LINE " << __LINE__ << ": SEND FAILED!!!" << endl;
 	}
-	else if (DEBUG)
+	else if (ERRORDEBUG)
 		cout << "sent " << count << "bytes" << endl;
 	delete[] buffer;
 }
@@ -216,7 +332,7 @@ void sendsource()
 	}
 	buffer[0]=ff::CMD_SOURCE; // copy message to buffer
 	memcpy(buffer+1, currentPacket->data, currentPacket->len);
-	if (DEBUG)
+	if (ERRORDEBUG)
 		cout << "Sending " << currentPacket->len+1 << " source bytes" << endl;
 
 	// send the data
@@ -225,7 +341,7 @@ void sendsource()
 		// on error disconnect client
 		free_ssl_auth();
 		//		clientConnected=0;
-		if (DEBUG)
+		if (ERRORDEBUG)
 			cerr << "LINE " << __LINE__ << ": SEND FAILED!!!" << endl;
 	}
 	delete[] buffer;
@@ -344,12 +460,13 @@ void senduserspacerules()
 		strcat(buffer+3, "\n");
 	}
 
-	if (DEBUG)
+	if (ERRORDEBUG)
 		cout << "Sending " << len+3 << " userspace rules bytes" << endl;
 
 	len=strlen(buffer+3)+3;
-	buffer[1]=(len-3)>>8; // length of sendbuffer, (buffer[0]=msgtype)
-	buffer[2]=(len-3)&255;
+	unsigned short *addr_len;
+	addr_len = (unsigned short *) (buffer+1);
+	*addr_len = htons( (short) len);
 
 	// send the data
 	if ((count=SSL_write(ssl_auth, buffer, len))<=0)
@@ -357,10 +474,10 @@ void senduserspacerules()
 		// on error disconnect the client
 		free_ssl_auth();
 		//		clientConnected=0;
-		if (DEBUG)
+		if (ERRORDEBUG)
 			cerr << "LINE " << __LINE__ << ": SEND FAILED!!!" << endl;
 	}
-	if (DEBUG)
+	if (ERRORDEBUG)
 		cout << "Sent " << count << " userspace rules bytes" << endl;
 	delete[] buffer;
 	if (ERRORDEBUG)
@@ -373,6 +490,7 @@ void sendrules()
 	char *buffer;
 	int count;
 	int len;
+	unsigned short * addr_len;
 	char *rules;
 
 	if (ERRORDEBUG)
@@ -394,8 +512,9 @@ void sendrules()
 		return;
 	}
 	buffer[0]=ff::CMD_RULES;
-	buffer[1]=len>>8; // size of message
-	buffer[2]=len&255;
+
+	addr_len= (unsigned short *) (buffer+1);
+	*addr_len=htons( (short) len);
 
 	memcpy(buffer+3, rules, len);
 	// send the data
@@ -530,8 +649,13 @@ tentry *inputUserspaceRule(char *buffer, packet *pack, int action)
 		system_log("Out of memory");
 		return 0;
 	}
-	if (buffer[11]||buffer[12]||buffer[13]||buffer[14]) // timeout is specified at buffer[11 - 14]
-		p->timeout=time(0)+(((((((unsigned char)buffer[11])<<8)+(unsigned char)buffer[12])<<8)+(unsigned char)buffer[13])<<8)+(unsigned char)buffer[14];
+
+	unsigned long *addr_timeout;
+	addr_timeout=(unsigned long *) (buffer+11);
+
+	// why this if ? time(0) is not 0?
+	if (*addr_timeout) //ntohl not needed
+	  p->timeout=time(0)+ntohl(*addr_timeout);
 	else
 		p->timeout=0;
 
@@ -719,8 +843,8 @@ void handle_cmd(int cmd, char *buffer)
 							  (buffer[ff::SB_IP_DST])?currentPacket->ip_dst:0,
 							  (buffer[ff::SB_PORT_SRC])?currentPacket->port_src:0,
 							  (buffer[ff::SB_PORT_DST])?currentPacket->port_dst:0,
-							  (buffer[ff::SB_INTERFACE_IN])?(char*)currentPacket->interface_in:0,
 							  (buffer[ff::SB_INTERFACE_OUT])?(char*)currentPacket->interface_out:0,
+							  (buffer[ff::SB_INTERFACE_IN])?(char*)currentPacket->interface_in:0,
 							  buffer[ff::SB_CONNTRACK], // connection tracking
 							  action);
 					// filter pending packets
@@ -791,12 +915,13 @@ void deleteUserspaceRule(int rulenum)
 void *rcvThread(void *parm)
 {
 	SSL *clientssl=(SSL*)parm;
-	short msgtype;
-	int result, len;
+	char msgtype;
+	int result;
+	unsigned short int len;
 	char buffer[20];
 	char username[50];
 	char password[50];
-	int temp;
+	int temp, in;
 	// is there an authenticated client? otherwise no commands except auth will be accepted
 	int clientAuthenticated=0;
 	// is there a client connected
@@ -806,19 +931,22 @@ void *rcvThread(void *parm)
 		cout << "new receive thread started" << endl;
 	while (clientConnected)
 	{
-		msgtype=-1;
+		msgtype=255;
 		if (ERRORDEBUG)
 			cout << __LINE__ << endl;
 
 		if (DEBUG)
 			cout << "awaiting message" << endl;
-		result=SSL_read(clientssl, &msgtype, 2); // first read message type
+		result=SSL_read(clientssl, &msgtype, 1); // first read message type
+		result=SSL_read(clientssl, &temp, 1); // Normally 0
+		//msgtype=msgtype>>8;
+		
 		if (clientAuthenticated && !ssl_auth) // error in one of the sends - close receiving thread too
 		{
 			clientAuthenticated=0;
 			clientConnected=0;
 		}
-		if ((result<=0) || (msgtype<=0)) //invalid?
+		if ((result<=0) ||/* (msgtype==255) ||*/ (msgtype==0)) //invalid?
 		{
 			// on error disconnect client
 			if (DEBUG)
@@ -833,7 +961,7 @@ void *rcvThread(void *parm)
 			switch (msgtype)
 			{
 			case ff::CMD_DELETE_RULE: // delete a iptables rule by number
-				if (!clientAuthenticated) break;
+			        if (!clientAuthenticated) break;
 				unsigned char chainid;
 				unsigned short rulenum;
 				if (ERRORDEBUG)
@@ -850,6 +978,7 @@ void *rcvThread(void *parm)
 					clientConnected=0;
 					break;
 				}
+				rulenum = ntohs (rulenum);
 				deleteRule(chainid, rulenum);
 				sendrules();
 				break;
@@ -857,24 +986,14 @@ void *rcvThread(void *parm)
 				pthread_mutex_lock(&lock_mutex);
 
 				// read username length
-				result=SSL_read(clientssl, &temp, 1);
+				result=SSL_read(clientssl, &len, 2);
 				if (result<=0)
 				{
 					pthread_mutex_unlock(&lock_mutex);
 					clientConnected=0;
 					break;
-				}
-				len=temp;
-				result=SSL_read(clientssl, &temp, 1);
-				if (result<=0)
-				{
-					clientConnected=0;
-					pthread_mutex_unlock(&lock_mutex);
-					break;
-				}
-				len+=temp<<8;
-
-				temp=0;
+					}
+				len=ntohs(len);
 				if (len<50) // length valid? --> read username
 				{
 					result=SSL_read(clientssl, username, len);
@@ -887,26 +1006,21 @@ void *rcvThread(void *parm)
 					username[len]=0;
 
 					// read password length
-					result=SSL_read(clientssl, &temp, 1);
+					result=SSL_read(clientssl, &len, 2);
 					if (result<=0)
 					{
 						clientConnected=0;
 						pthread_mutex_unlock(&lock_mutex);
 						break;
 					}
-					len=temp;
-					result=SSL_read(clientssl, &temp, 1);
-					if (result<=0)
-					{
-						clientConnected=0;
-						pthread_mutex_unlock(&lock_mutex);
-						break;
-					}
-					len+=temp<<8;
-					temp=0;
+					len=ntohs(len);
 					if (len<50) // length valid? --> read password
 					{
-						result=SSL_read(clientssl, password, len);
+                        // we want to support zero length passwords
+                        // even if they should not be used
+                        result=1;
+                        if (len>0)
+							result=SSL_read(clientssl, password, len);
 						if (result<=0)
 						{
 							clientConnected=0;
@@ -965,6 +1079,7 @@ void *rcvThread(void *parm)
 					clientConnected=0;
 					break;
 				}
+				rulenum = ntohs( rulenum );
 				deleteUserspaceRule(rulenum);
 				senduserspacerules();
 				break;
@@ -1151,36 +1266,50 @@ packet* createPacket(ipq_packet_msg_t *msg)
 	p->port_src=0;
 	p->port_dst=0;
 
-	p->packet_id=msg->packet_id;
-	p->arrived=msg->timestamp_sec;
+	p->packet_id = /*ntohl*/( msg->packet_id );
+	p->arrived = /*ntohl*/( msg->timestamp_sec );
 	if (!p->arrived) // workaround for unspecified time in ipq codes
 		p->arrived=time(NULL);
-	memcpy(p->interface_in, msg->indev_name, IFNAMSIZ);
-	memcpy(p->interface_out, msg->outdev_name, IFNAMSIZ);
-	p->len=(short)msg->data_len;
+//	memcpy(p->interface_in, msg->indev_name, IFNAMSIZ);
+//	memcpy(p->interface_out, msg->outdev_name, IFNAMSIZ);
+    // indev means the interface where the packet came from
+    // this is only set in packets coming to the INPUT chain
+    // and there the _destination_ part uses this device.
+    // we wont know anything about the device of the source
+    // as this is on another computer
+    // outdev vice versa.
+	memcpy(p->interface_out, msg->indev_name, IFNAMSIZ);
+	memcpy(p->interface_in, msg->outdev_name, IFNAMSIZ);
+	p->len = /*ntohs(*/ (short)msg->data_len /*)*/;
+
+	// ip addr could be char[4] and not uint.
 	p->ip_src=iph->saddr;
 	p->ip_dst=iph->daddr;
-	p->protocol=iph->protocol;
+	// iph->protocol has type char
+	p->protocol= iph->protocol ;
 
 	if (iph->protocol==ff::PROT_TCP) // copy ports and flags for tcp
 	{
 		tcph=(struct tcphdr*)((char*)iph+sizeof(struct iphdr));
-		p->port_src=(tcph->source>>8)+((tcph->source&255)<<8);
-		p->port_dst=(tcph->dest>>8)+((tcph->dest&255)<<8);
+		p->port_src=ntohs( tcph->source );
+		p->port_dst=ntohs( tcph->dest );
+		// keeped. Must run like that.
 		p->tcp_flags=(int)tcph->fin+(((int)tcph->syn)<<1)+(((int)tcph->ack)<<2)+(((int)tcph->rst)<<3);   //*((short*)(((char*)(&tcph->ack_seq))+4));
 	}
 	else if (iph->protocol==ff::PROT_UDP) // copy ports for udp
 	{
 		udph=(struct udphdr*)((char*)iph+sizeof(struct iphdr));
-		p->port_src=(udph->source>>8)+((udph->source&255)<<8);
-		p->port_dst=(udph->dest>>8)+((udph->dest&255)<<8);
+		p->port_src=ntohs( udph->source );
+		p->port_dst=ntohs( udph->dest );
 	}
 	else if (iph->protocol==ff::PROT_ICMP) // copy icmp type for icmp
 	{
 		icmph=(struct icmphdr*)((char*)iph+sizeof(struct iphdr));
+		// icmph->type has type char .
 		p->icmp_type=icmph->type;
 	}
 
+	// char and char[8]
 	p->mac_addrlen=msg->hw_addrlen;
 	memcpy(p->mac_addr, msg->hw_addr, 8);
 
@@ -1193,6 +1322,7 @@ packet* createPacket(ipq_packet_msg_t *msg)
 		return 0;
 	}
 
+	// keep data in network order
 	memcpy(p->data, msg->payload, p->len);
 	p->programname=0;
 	if (msg->hook==ff::CHAIN_INPUT) // get programname for tcp and udp if chain is INPUT or OUTPUT
@@ -1206,7 +1336,7 @@ packet* createPacket(ipq_packet_msg_t *msg)
 			p->programname=get_program_name(p->protocol, p->ip_src, p->port_src);
 	}
 
-	p->hook=msg->hook;
+	p->hook=/*ntohl*/( msg->hook );
 
 	if (ERRORDEBUG)
 		cout << "-createpacket" << endl;
@@ -1339,19 +1469,12 @@ void *kernelThread(void *parm)
 	int status;
 	unsigned long clientip;
 	unsigned char buf[BUFSIZE];
-	int c1, c2, c3, c4;
+	struct in_addr *addr;
 
-	// get ip of accepted client
-	if (sscanf(Config_client_ip, "%d.%d.%d.%d", &c1, &c2, &c3, &c4)==4)
-	{
-		clientip=((unsigned long)c4<<24)+((unsigned long)c3<<16)+((unsigned long)c2<<8)+((unsigned long)c1); // ip address as number
-	}
-	else
-	{
-		cerr << "Invalid client_ip" << endl;
-		system_log("Invalid client ip in config file");
-		return 0;
-	}
+	addr = dotted_to_addr( Config_client_ip );
+	clientip = (unsigned long) addr->s_addr;
+
+	cout << "Client IP: " << clientip << endl;
 
 	// set queue to maximum QUEUE_LENGTH packets
 	// if queue is full packets are dropped, add returns false --> refuse packet
@@ -1403,7 +1526,8 @@ void *kernelThread(void *parm)
 				cout << __LINE__ << endl;
 
 			int verdict;
-			// is it a fireflier packet ?
+			// is it a fireflier packet ? 
+			cout << "Paquet ip: " << pack->ip_dst << " port src : " << pack->port_src << "dst " << pack->port_dst << endl;
 			if (((pack->ip_dst==clientip) && (pack->port_src==Config_port)) ||
 				((pack->ip_src==clientip) && (pack->port_dst==Config_port)))
 				verdict=ff::ACTION_ACCEPT;
@@ -2060,7 +2184,7 @@ void loadconfig(char *filename)
 						strncpy(temp, line+valuestart, valueend-valuestart);
 						temp[valueend-valuestart]=0;
 						if (atoi(temp)!=0)
-							Config_port=atoi(temp);
+							Config_port=/*htons*/((short) atoi(temp));
 						else
 							cerr << "Config syntax error in line: " << linenr << " - port invalid" << endl;
 					} // should we create the iptables queue rules ?
