@@ -189,15 +189,15 @@ int inputRule(unsigned int chainid, int protocol, unsigned long int srcip, unsig
     {
         sprintf(temp, " --source-port %d", srcport);
         strcat(cmd, temp);
-        sprintf(temp, " --destination-port %d", srcport);
-        strcat(revcmd, temp);
+//        sprintf(temp, " --destination-port %d", srcport);
+//        strcat(revcmd, temp);
     }
     if (dstport!=0) // match destination port
     {
         sprintf(temp, " --destination-port %d", dstport);
         strcat(cmd, temp);
-        sprintf(temp, " --source-port %d", dstport);
-        strcat(revcmd, temp);
+//        sprintf(temp, " --source-port %d", dstport);
+//        strcat(revcmd, temp);
     }
 
     if (interface_in!=0) // match incoming interface names
@@ -238,7 +238,7 @@ int inputRule(unsigned int chainid, int protocol, unsigned long int srcip, unsig
 //    printf("%s\n", cmd);
     if (conntrack) // only if connection tracking wanted use inverted rule
     {
-	system(revcmd);
+		system(revcmd);
     }
     system(cmd); // insert rule
     return 1;
@@ -320,15 +320,15 @@ void deleteRule(unsigned int chainid, int protocol, unsigned long int srcip, uns
     {
         sprintf(temp, " --source-port %d", srcport);
         strcat(cmd, temp);
-        sprintf(temp, " --destination-port %d", srcport);
-        strcat(revcmd, temp);
+//        sprintf(temp, " --destination-port %d", srcport);
+//        strcat(revcmd, temp);
     }
     if (dstport!=0) // destination port of packets
     {
         sprintf(temp, " --destination-port %d", dstport);
         strcat(cmd, temp);
-        sprintf(temp, " --source-port %d", dstport);
-        strcat(revcmd, temp);
+//        sprintf(temp, " --source-port %d", dstport);
+//        strcat(revcmd, temp);
     }
 
     if (interface_in!=0) // incoming interface
